# Entity Framework Core 3.0
*31/12/2019 &sdot; Marc Chevaldonn*  

---
Entity Framework (EF) Core est un ORM (Object-Relational Mapper) qui permet aux dveloppeurs .NET de grer de manire simple, lgre et extensible, des bases de donnes.
EF permet de grer de nombreux *providers* (SQL Server, SQLite, Cosmos, ...) de manire transparente.
EF vous permet galement de mettre  jour vos bases de donnes et d'excuter des requtes sans avoir  crire la moindre requte SQL. Vous pouvez passer par LINQ to SQL qui apportera plus de lisibilit et permettra au compilateur de vous aider  dtecter vos erreurs.
  
---
*Note:*  
Diffrentes solutions existent avec EF pour grer une base de donnes dont le modle existe par exemple. Dans ces exemples, je ne traiterai que la partie *Code First*, c'est--dire le cas o le modle est cr  partir de vos classes.
  
---
## Plan
Les exemples sont organiss selon le plan suivant:  
1. *Fundamentals* :  
Dans cette partie, je donnerai quelques notions pour se connecter  une base  l'aide de chane de connection (*connection strings*), comment utiliser des *providers de tests...*.
Il s'agira en consquence d'exemples simples manquants d'explications sur certains points, car ils seront prsents plus tard.
   * **ex_041_001 : Connection Strings** : montre comment utiliser une chane de connexion SQL Server ou SQLite.
2. *Model* :  
Ce chapitre s'attardera sur le lien entre le modle et la base de donnes. En effet, avec EF, l'accs aux donnes se fait via le modle, c'est--dire l'ensemble de vos classes (qui seront relies  des tables cres plus ou moins automatiquement)
 ainsi qu'un contexte (```DbContext```) qui reprsentera une session de connexion avec votre (ou vos) base(s) de donnes.
   Je prsenterai en consquence tout d'abord comment crire des classes pour votre modle, puis comment crire les diffrentes relations classiques (aggrgation, *one to one*, *one to many*, *many to many*, mais aussi les dictionnaires), comment grer les hritages entre classes du modle dans la base de donnes, etc.
3. *Schemas and migrations* :
   Le but de ce chapitre sera de vous montrer comment garder votre modle et votre base de donnes synchroniss.
4. *Querying (LINQ to SQL) and saving data* :
   *Language INtegrated Query* (LINQ) est un outil de requte sur collections et sa version LINQ to SQL vous permet de passer trs facilement  un systme de requtes sur les bases de donnes.
Les requtes LINQ sont automatiquement traduites en requtes SQL, vous vitant ainsi d'avoir  crire vos requtes vous-mmes. Elles sont ds lors beaucoup plus lisibles et faciles  crire.
Ce chapitre prsente comment charger des donnes, raliser du filtrage, de manire synchrone ou asynchrone, etc.
   Il montre bien sr galement comment raliser le symtrique : mettre  jour, supprimer ou ajouter de nouvelles donnes dans la base.
5. *Database providers* :
   EF vous permet de grer votre base de donnes indpendamment du *provider*. Ce chapitre montrera donc comment utiliser diffrents providers parmi lesquels Microsoft SQL Server, SQLite ou encore InMemory dont le but est de permettre de tester la base en mmoire, sans passer par un *provider*.
   
---

## Quelle version utiliser ?
Ces exemples sont crits pour .NET Core 3.0, mais vous pouvez utiliser EF Core avec diffrents types projets. Voici les recommendations actuelles de Microsoft quant  l'utilisation des version d'EF.
   
|**EF Core**     |**1.x** |**2.x**    |**3.x**
|----------------|--------|-----------|---------------
|.NET Standard   |1.3     |2.0        |2.1
|.NET Core       |1.0     |2.0        |3.0
|.NET Framework  |4.5.1   |4.7.2      |(not supported)
|Mono            |4.6     |5.4        |6.4
|Xamarin.iOS     |10.0    |10.14      |12.16
|Xamarin.Android |7.0     |8.0        |10.0
|UWP             |10.0    |10.0.16299 |to be defined
|Unity           |2018.1  |2018.1     |to be defined

   *Comment lire ce tableau ?*
   
Si vous voulez utiliser EF Core 3.0 avec une bibliothque de classes crites en .NET Standard, celle-ci doit utiliser au moins .NET Standard 2.1.
   
Si vous voulez utiliser EF Core 3.0 avec un projet Xamarin.iOS, celui-ci doit tre au moins en version 12.16.
   
Si vous voulez utiliser EF Core dans une application UWP, vous ne pouvez pour le moment utiliser que EF Core 1.x ou 2.x.
  
---
   *Note :*
   
Je n'ai pas l'intention de mettre  jour les exemples pour Entity Framework 6 ou pour .NET Framework, puisque la version 5 du framework va unifier .NET Framework et .NET Core. En consquence, EF Core sera la nouvelle "norme".

    
---
## Comment commencer ?
Un petit tutoriel rapide pour savoir comment crer un projet...
##### Prrequis
Il vous faut au moins la **[version 3.0 du SDK de .NET Core](https://dotnet.microsoft.com/download)**, mais celle-ci est certainement dj installe si vous avez install Visual Studio 2019 16.3 ou plus.
##### Crez un nouveau projet
Vous pouvez ensuite crer un nouveau projet .NET Core 3.x, pour cela :
* lancez Visual Studio
* crez un nouveau projet de type **Console App (.NET Core)** en C#

##### Installez EntityFramework Core
Pour ce tutoriel, nous pouvons utiliser SqlServer comme *provider*.
* Pour cela, cliquez droit sur le projet, puis slectionnez *Grer les packages NuGet...*.
* Sous l'onglet *Parcourir*, dans la barre de recherche, rentrez *Microsoft.EntityFrameworkCore*
* Slectionnez le premier nuget dans sa version la plus rcente et lancez l'installation.
* Rptez les deux dernires oprations pour les packages :
  * *Microsoft.EntityFrameworkCore.Design*
  * *Microsoft.EntityFrameworkCore.SqlServer*
  * *Microsoft.EntityFrameworkCore.SqlServer.Design*

##### Crez un modle
* Ajoutez une nouvelle classe ```Nounours``` au projet.
* Ajoutez le code suivant  cette classe :
```csharp
using System;

namespace tutoRapideEFCore
{
    class Nounours
    {
        public int Id { get; set; }
        public string Nom { get; set; }
        public DateTime Naissance { get; set; }
        public int NbPoils { get; set; }
    }
}
```
* Ajoutez une nouvelle classe ```NounoursContext``` qui servira de contexte  notre modle.
* Ajoutez le code suivante  cette classe :
```csharp
using Microsoft.EntityFrameworkCore;

namespace tutoRapideEFCore
{
    class NounoursContext : DbContext
    {
        public DbSet<Nounours> Nounours { get; set; }

        protected override void OnConfiguring(DbContextOptionsBuilder options)
            => options.UseSqlServer(@"Server=(localdb)\mssqllocaldb;Database=myFirstDatabase.mdf;Trusted_Connection=True;");
    }
}

```
##### Crez la base de donnes
* Ouvrez la *Console du Gestionnaire de package*, pour cela, dirigez-vous dans le menu *Outils*, puis *Gestionnaire de package NuGet*, puis *Console du Gestionnaire de package*.
* Dans la console que vous venez d'ouvrir, dplacez-vous dans le dossier du projet (*eg* si votre projet s'apelle **tutoRapideEFCore**) :
```
cd tutoRapideEFCore
```
* tapez ensuite les commandes suivantes :
```
dotnet ef migrations add myFirstMigration
dotnet ef database update 
```
*Note*:
   
Si vous n'avez pas install correctement EntityFrameworkCore, il vous faudra peut-tre utiliser galement :
   
```dotnet tool install --global dotnet-ef```

##### Utilisez votre base de donnes via Entity Framework Core
* Editez *Program.cs* et ajoutez le code suivant :
```csharp
static void Main(string[] args)
{
    Nounours chewie = new Nounours { Nom = "Chewbacca", Naissance = new DateTime(1977, 5, 27), NbPoils = 1234567 };
    Nounours yoda = new Nounours { Nom = "Yoda", Naissance = new DateTime(1980, 5, 21), NbPoils = 3 };
    Nounours ewok = new Nounours { Nom = "Ewok", Naissance = new DateTime(1983, 5, 25), NbPoils = 3456789 };
    Nounours c3po = new Nounours { Nom = "C3PO", Naissance = new DateTime(1977, 5, 27), NbPoils = 0 };

    using (var context = new NounoursContext())
    {
        // Cre des nounours et les insre dans la base
        Console.WriteLine("Creates and inserts new Nounours");
        context.Add(chewie);
        context.Add(yoda);
        context.Add(ewok);
        context.Add(c3po);
        context.SaveChanges();
    }
}
```
Maintenant, lorsque vous lancerez l'application, la base de donnes contiendra 3 nounours. La base cre automatiquement des identifiants pour chaque Nounours.
* Editez *Program.cs* pour rajouter les lignes suivantes  la fin de la mthode ```Main``` :
```csharp
// Lit le premier nounours de la base dont le nom commence par 'e'
Console.WriteLine("Creates and executes a query retrieving the first Nounours of the database whose name starts with an \"e\":");
var eNounours = context.Nounours
    .Where(n => n.Nom.StartsWith("e"))
    .First();
Console.WriteLine($"{eNounours.Nom} (born in {eNounours.Naissance.Year})");
```
Cette requte LINQ vous permet de lire le premier nounours de la base de nounours tris par ordre alphabtique de noms.  
Ceci ncessite de rajouter au-dbut du fichier *Program.cs* la ligne suivante :  
```csharp
using System.Linq;
```
* Editez *Program.cs* pour rajouter les lignes suivantes  la fin de la mthode ```Main``` :
```csharp
// Met  jour le nom du second nounours de la base
Console.WriteLine("Updates the name of the second nounours");
eNounours.Nom = "Wicket";
context.SaveChanges();
```
Cette partie du code montre comment mettre  jour un lment de la base de donnes.
* Editez *Program.cs* pour rajouter les lignes suivantes  la fin de la mthode ```Main``` :
```csharp
// rcupre le nounours qui n'en est pas un et le supprime de la base
Console.WriteLine("Deletes one item from de database");
var droid = context.Nounours
    .SingleOrDefault(n => n.Nom.Equals("C3PO"));
context.Remove(droid);
context.SaveChanges();
```
Cette partie du code montre comment supprimer un lment de la base de donnes.   
   
Voici un rcapitulatif du fichier *Program.cs* :
```csharp
using System;
using System.Linq;

namespace tutoRapideEFCore
{
    class Program
    {
        static void Main(string[] args)
        {
            Nounours chewie = new Nounours { Nom = "Chewbacca", Naissance = new DateTime(1977, 5, 27), NbPoils = 1234567 };
            Nounours yoda = new Nounours { Nom = "Yoda", Naissance = new DateTime(1980, 5, 21), NbPoils = 3 };
            Nounours ewok = new Nounours { Nom = "Ewok", Naissance = new DateTime(1983, 5, 25), NbPoils = 3456789 };
            Nounours c3po = new Nounours { Nom = "C3PO", Naissance = new DateTime(1977, 5, 27), NbPoils = 0 };

            using (var context = new NounoursContext())
            {
                // Cre des nounours et les insre dans la base
                Console.WriteLine("Creates and inserts new Nounours");
                context.Add(chewie);
                context.Add(yoda);
                context.Add(ewok);
                context.Add(c3po);
                context.SaveChanges();

                // Lit le premier nounours de la base dont le nom commence par 'e'
                Console.WriteLine("Creates and executes a query retrieving the first Nounours of the database whose name starts with an \"e\":");
                var eNounours = context.Nounours
                    .Where(n => n.Nom.StartsWith("e"))
                    .First();
                Console.WriteLine($"{eNounours.Nom} (born in {eNounours.Naissance.Year})");

                // Met  jour le nom du second nounours de la base
                Console.WriteLine("Updates the name of the second nounours");
                eNounours.Nom = "Wicket";
                context.SaveChanges();

                // rcupre le nounours qui n'en est pas un et le supprime de la base
                Console.WriteLine("Deletes one item from de database");
                var droid = context.Nounours
                    .SingleOrDefault(n => n.Nom.Equals("C3PO"));
                context.Remove(droid);
                context.SaveChanges();
            }
        }
    }
}
```
* Excutez votre application pour vrifier son bon fonctionnement.

##### Vrifiez le contenu de la base de donnes avec l'Explorateur d'objets SQL Server
Vous pouvez vrifier le contenu de votre base en utilisant l'*Explorateur d'objets SQL Server*.
* Pour cela, allez dans le menu *Affichage* puis *Explorateur d'objets SQL Server*.
* Dployez dans l'*Explorateur d'objets SQL Server* :
  *  *SQL Server*, 
  *  puis *(localdb)\MSSQLLocalDB ...*, 
  *  puis *Bases de donnes*
  *  puis celle portant le nom de votre migration, dans mon cas : *myFirstDatabase.Nounours.mdf*
  *  puis *Tables*   
  *  Faites un clic droit sur la table *dbo.Nounours* puis choisissez *Afficher les donnes*
  *  Vous devriez maintenant pouvoir voir les donnes suivantes dans le tableau :  
 
  |Id   |Nom  |Naissance |NbPoils
  |---|---|---|---
  |1|Chewbacca|27/05/1977 00:00:00|1234567
  |2|Yoda|21/05/1980 00:00:00|3
  |3|Wicket|25/05/1983 00:00:00|3456789

Vous pouvez constater que l'Ewok a bien t renomm Wicket, et que C3PO a bien t supprim.  
Notez qu'il est galement possible d'utiliser l'*Explorateur d'objets SQL Server* pour ajouter, modifier ou supprimer des donnes dans les tables.